# vim:set noet ts=4:
#
# ibus-anthy - The Anthy engine for IBus
#
# Copyright (c) 2010-2016 Takao Fujiwara <takao.fujiwara1@gmail.com>
# Copyright (c) 2010-2016 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# This file was imported from im-dict-ja
# im-dict-ja - Japanese word dictionary set for input method
# http://github.com/fujiwarat/im-dict-ja
あっとまーく #T35*0 ＠
いちのじ #T35*0 ゝ
いちのじ #T35*0 ゞ
いちのじ #T35*0 ヽ
いちのじ #T35*0 ヾ
いてざ #T35*0 ♐
うおざ #T35*0 ♓
えもじ #T35*0 〄
えもじ #T35*0 ☎
えもじ #T35*0 〠
えん #T35*0 ￥
えんざん #T35*0 ±
えんざん #T35*0 X
えんざん #T35*0 ÷
えんざん #T35*0 *
えんざん #T35*0 ≠
えんざん #T35*0 ≦
えんざん #T35*0 ≧
えんざん #T35*0 ∞
えんざん #T35*0 ∴
えんざん #T35*0 ∵
えんざん #T35*0 ＜
えんざん #T35*0 ＞
えんざん #T35*0 ＝
えんざん #T35*0 −
えんざん #T35*0 ＋
えんざん #T35*0 ≒
えんざん #T35*0 ≡
えんざん #T35*0 ≪
えんざん #T35*0 ≫
えんざん #T35*0 ∽
えんざん #T35*0 ∈
えんざん #T35*0 ∋
えんざん #T35*0 ⊆
えんざん #T35*0 ⊇
えんざん #T35*0 ⊂
えんざん #T35*0 ⊃
えんざん #T35*0 ∪
えんざん #T35*0 ∩
えんざん #T35*0 ∧
えんざん #T35*0 ∨
えんざん #T35*0 ¬
えんざん #T35*0 ⇒
えんざん #T35*0 ⇔
えんざん #T35*0 ∀
えんざん #T35*0 ∃
えんざん #T35*0 ∠
えんざん #T35*0 ⊥
えんざん #T35*0 ⌒
えんざん #T35*0 ∂
えんざん #T35*0 ∇
えんざん #T35*0 √
えんざん #T35*0 ∫
えんざん #T35*0 ∬
おうしざ #T35*0 ♉
おとめざ #T35*0 ♍
おひつじざ #T35*0 ♈
おんぷ #T35*0 ♪
おんぷ #T35*0 𝄞
おんぷ #T35*0 𝄢
おんぷ #T35*0 𝄻
おんぷ #T35*0 𝄼
おんぷ #T35*0 𝄽
おんぷ #T35*0 𝄾
おんぷ #T35*0 🎵
おんぷ #T35*0 🎶
かっこ #T35*0 「
かっこ #T35*0 」
かっこ #T35*0 『
かっこ #T35*0 』
かっこ #T35*0 〈
かっこ #T35*0 〉
かっこ #T35*0 《
かっこ #T35*0 》
かっこ #T35*0 〔
かっこ #T35*0 〕
かっこ #T35*0 ［
かっこ #T35*0 ］
かっこ #T35*0 【
かっこ #T35*0 】
かっこ #T35*0 （
かっこ #T35*0 ）
かっこ #T35*0 ｛
かっこ #T35*0 ｝
かっこ #T35*0 ‘
かっこ #T35*0 ’
かっこ #T35*0 “
かっこ #T35*0 ”
かっこ #T35*0 〝
かっこ #T35*0 〟
かっこすうじ #T35*0 ⑴
かっこすうじ #T35*0 ⑵
かっこすうじ #T35*0 ⑶
かっこすうじ #T35*0 ⑷
かっこすうじ #T35*0 ⑸
かっこすうじ #T35*0 ⑹
かっこすうじ #T35*0 ⑺
かっこすうじ #T35*0 ⑻
かっこすうじ #T35*0 ⑼
かっこすうじ #T35*0 ⑽
かっこすうじ #T35*0 ⒂
かっこすうじ #T35*0 ⒇
かっころーまじ #T35*0 ⒜
かっころーまじ #T35*0 ⒝
かっころーまじ #T35*0 ⒞
かっころーまじ #T35*0 ⒟
かっころーまじ #T35*0 ⒠
かっころーまじ #T35*0 ⒡
かっころーまじ #T35*0 ⒢
かっころーまじ #T35*0 ⒣
かっころーまじ #T35*0 ⒤
かっころーまじ #T35*0 ⒥
かっころーまじ #T35*0 ⒦
かっころーまじ #T35*0 ⒧
かっころーまじ #T35*0 ⒨
かっころーまじ #T35*0 ⒩
かっころーまじ #T35*0 ⒪
かっころーまじ #T35*0 ⒫
かっころーまじ #T35*0 ⒬
かっころーまじ #T35*0 ⒭
かっころーまじ #T35*0 ⒮
かっころーまじ #T35*0 ⒯
かっころーまじ #T35*0 ⒰
かっころーまじ #T35*0 ⒱
かっころーまじ #T35*0 ⒲
かっころーまじ #T35*0 ⒳
かっころーまじ #T35*0 ⒴
かっころーまじ #T35*0 ⒵
かにざ #T35*0 ♋
かぶしきがいしゃ #T35*0 ㍿
かぶしきがいしゃ #T35*0 ㈱
かぶしきがいしゃ #T35*0 ㏍
きごう #T35*0 ©
きごう #T35*0 ®
きごう #T35*0 ℗
きごう #T35*0 ℠
きごう #T35*0 ™
きごう #T35*0 ⁈
きごう #T35*0 ⁉
きごう #T35*0 ⁇
きごう #T35*0 ‼
きゅうかな #T35*0 ゐ
きゅうかな #T35*0 ヰ
きゅうかな #T35*0 ヸ
きゅうかな #T35*0 ゑ
きゅうかな #T35*0 ヱ
きゅうかな #T35*0 ヹ
きゅうかな #T35*0 ヺ
きゅうかな #T35*0 ゔ
きゅうかな #T35*0 ヷ
きゅうかな #T35*0 ヴ
きろ #T35*0 ㎞
きろ #T35*0 ㎏
きろ #T35*0 ㎅
きろ #T35*0 ㌔
きろ #T35*0 ㌖
きろ #T35*0 ㌕
きろぐらむ #T35*0 ㎏
きろぐらむ #T35*0 ㌕
くのじ #T35*0 〳
くのじ #T35*0 〵
くのじ #T35*0 〴
ぐらむ #T35*0 ｇ
ぐらむ #T35*0 ㌘
けいけい #T35*0 ㏍
けいせん #T35*0 ─
けいせん #T35*0 │
けいせん #T35*0 ┌
けいせん #T35*0 ┐
けいせん #T35*0 ┘
けいせん #T35*0 └
けいせん #T35*0 ├
けいせん #T35*0 ┬
けいせん #T35*0 ┤
けいせん #T35*0 ┴
けいせん #T35*0 ┼
けいせん #T35*0 ━
けいせん #T35*0 ┃
けいせん #T35*0 ┏
けいせん #T35*0 ┓
けいせん #T35*0 ┛
けいせん #T35*0 ┗
けいせん #T35*0 ┣
けいせん #T35*0 ┳
けいせん #T35*0 ┫
けいせん #T35*0 ┻
けいせん #T35*0 ╋
けいせん #T35*0 ￣
けいせん #T35*0 ︳
けいせん #T35*0 ー
けいせん #T35*0 ︱
こと #T35*0 ヿ
こぴーらいと #T35*0 ©
こぴーらいと #T35*0 ℗
こめ #T35*0 ※
さそりざ #T35*0 ♏
さんかく #T35*0 ▲
さんかく #T35*0 △
さんかく #T35*0 ▼
さんかく #T35*0 ▽
さーびすまーく #T35*0 ℠
しかく #T35*0 □
しかく #T35*0 ■
しかく #T35*0 ◇
しかく #T35*0 ◆
ししざ #T35*0 ♌
しゃーぷ #T35*0 ♯
しょうひょう #T35*0 ©
しょうひょう #T35*0 ℗
しょうりゃくもじ #T35*0 №
しょうりゃくもじ #T35*0 ㏍
しょうりゃくもじ #T35*0 ℡
しょうりゃくもじ #T35*0 ㍿
しょうりゃくもじ #T35*0 ㌀
しょうりゃくもじ #T35*0 ㌞
しょうりゃくもじ #T35*0 ㌪
しょうりゃくもじ #T35*0 ㌱
しょうりゃくもじ #T35*0 ㍇
しょうりゃくもじ #T35*0 ㈰
しょうりゃくもじ #T35*0 ㈪
しょうりゃくもじ #T35*0 ㈫
しょうりゃくもじ #T35*0 ㈬
しょうりゃくもじ #T35*0 ㈭
しょうりゃくもじ #T35*0 ㈮
しょうりゃくもじ #T35*0 ㈯
しょうりゃくもじ #T35*0 ㉀
しょうりゃくもじ #T35*0 ㈷
しょうりゃくもじ #T35*0 ㉂
しょうりゃくもじ #T35*0 ㉃
しょうりゃくもじ #T35*0 ㈹
しょうりゃくもじ #T35*0 ㈺
しょうりゃくもじ #T35*0 ㈱
しょうりゃくもじ #T35*0 ㈾
しょうりゃくもじ #T35*0 ㈴
しょうりゃくもじ #T35*0 ㈲
しょうりゃくもじ #T35*0 ㈻
しょうりゃくもじ #T35*0 ㈶
しょうりゃくもじ #T35*0 ㈳
しょうりゃくもじ #T35*0 ㈵
しょうりゃくもじ #T35*0 ㈼
しょうりゃくもじ #T35*0 ㈽
しょうりゃくもじ #T35*0 ㈿
しょうりゃくもじ #T35*0 ㈸
しょうりゃくもじ #T35*0 ㊤
しょうりゃくもじ #T35*0 ㊥
しょうりゃくもじ #T35*0 ㊦
しょうりゃくもじ #T35*0 ㊧
しょうりゃくもじ #T35*0 ㊨
しょうりゃくもじ #T35*0 ㊩
しょうりゃくもじ #T35*0 ㊖
しょうりゃくもじ #T35*0 ㊝
しょうりゃくもじ #T35*0 ㊘
しょうりゃくもじ #T35*0 ㊞
しょうりゃくもじ #T35*0 ㊙
しょうりゃくもじ #T35*0 ＠
しょうりゃくもじ #T35*0 ©
しょうりゃくもじ #T35*0 ®
しょうりゃくもじ #T35*0 ℗
しょうりゃくもじ #T35*0 ℠
しょうりゃくもじ #T35*0 ™
しょうわ #T35*0 ㍼
じす #T35*0 〄
すうがく #T35*0 ＋
すうがく #T35*0 −
すうがく #T35*0 ±
すうがく #T35*0 X
すうがく #T35*0 ÷
すうがく #T35*0 *
すうがく #T35*0 ＝
すうがく #T35*0 ≠
すうがく #T35*0 ≒
すうがく #T35*0 ≡
すうがく #T35*0 ＜
すうがく #T35*0 ＞
すうがく #T35*0 ≦
すうがく #T35*0 ≧
すうがく #T35*0 ≪
すうがく #T35*0 ≫
すうがく #T35*0 ∞
すうがく #T35*0 ∽
すうがく #T35*0 ∝
すうがく #T35*0 ∴
すうがく #T35*0 ∵
すうがく #T35*0 ∈
すうがく #T35*0 ∋
すうがく #T35*0 ⊆
すうがく #T35*0 ⊇
すうがく #T35*0 ⊂
すうがく #T35*0 ⊃
すうがく #T35*0 ∪
すうがく #T35*0 ∩
すうがく #T35*0 ∧
すうがく #T35*0 ∨
すうがく #T35*0 ¬
すうがく #T35*0 ⇒
すうがく #T35*0 ⇔
すうがく #T35*0 ∀
すうがく #T35*0 ∃
すうがく #T35*0 ∠
すうがく #T35*0 ⊥
すうがく #T35*0 ⌒
すうがく #T35*0 ∂
すうがく #T35*0 ∇
すうがく #T35*0 √
すうがく #T35*0 ∫
すうがく #T35*0 ∬
すうがく #T35*0 ∮
すうがく #T35*0 ∟
すうがく #T35*0 ⊿
せいざ #T35*0 ♈
せいざ #T35*0 ♉
せいざ #T35*0 ♊
せいざ #T35*0 ♋
せいざ #T35*0 ♌
せいざ #T35*0 ♍
せいざ #T35*0 ♎
せいざ #T35*0 ♏
せいざ #T35*0 ♐
せいざ #T35*0 ♑
せいざ #T35*0 ♒
せいざ #T35*0 ♓
せくしょん #T35*0 §
せんち #T35*0 ㎝
せんち #T35*0 ㌢
せんと #T35*0 ¢
せんと #T35*0 ㌣
たいしょう #T35*0 ㍽
たてかな #T35*0 ぁ
たてかな #T35*0 ぃ
たてかな #T35*0 ぅ
たてかな #T35*0 ぇ
たてかな #T35*0 ぉ
たてかな #T35*0 っ
たてかな #T35*0 ゃ
たてかな #T35*0 ゅ
たてかな #T35*0 ょ
たてかな #T35*0 ゎ
たてかな #T35*0 ァ
たてかな #T35*0 ィ
たてかな #T35*0 ゥ
たてかな #T35*0 ェ
たてかな #T35*0 ォ
たてかな #T35*0 ッ
たてかな #T35*0 ャ
たてかな #T35*0 ュ
たてかな #T35*0 ョ
たてかな #T35*0 ヮ
たてかな #T35*0 ヵ
たてかな #T35*0 ヶ
たてきごう #T35*0 ‐
たてきごう #T35*0 〜
たてきごう #T35*0 ‖
たてきごう #T35*0 ｜
たてきごう #T35*0 ︰
たてきごう #T35*0 ︵
たてきごう #T35*0 ︶
たてきごう #T35*0 ︹
たてきごう #T35*0 ︺
たてきごう #T35*0 ［
たてきごう #T35*0 ］
たてきごう #T35*0 ︷
たてきごう #T35*0 ︸
たてきごう #T35*0 ︿
たてきごう #T35*0 ﹀
たてきごう #T35*0 ︽
たてきごう #T35*0 ︾
たてきごう #T35*0 ﹁
たてきごう #T35*0 ﹂
たてきごう #T35*0 ﹃
たてきごう #T35*0 ﹄
たてきごう #T35*0 ︻
たてきごう #T35*0 ︼
たてきごう #T35*0 ＝
たてぼう #T35*0 ‖
たてぼう #T35*0 ｜
たてぼう #T35*0 ＼
たてぼう #T35*0 ／
たんい #T35*0 ￥
たんい #T35*0 ＄
たんい #T35*0 ¢
たんい #T35*0 £
たんい #T35*0 ％
たんい #T35*0 ‰
たんい #T35*0 ㎜
たんい #T35*0 ㎟
たんい #T35*0 ㎝
たんい #T35*0 ㎠
たんい #T35*0 ㎤
たんい #T35*0 ｍ
たんい #T35*0 ㎡
たんい #T35*0 ㎥
たんい #T35*0 ㎞
たんい #T35*0 ㎢
たんい #T35*0 ㎎
たんい #T35*0 ｇ
たんい #T35*0 ㎏
たんい #T35*0 ㏄
たんい #T35*0 ㎖
たんい #T35*0 ㎗
たんい #T35*0 ℓ
たんい #T35*0 ㎘
たんい #T35*0 ㎳
たんい #T35*0 ㎲
たんい #T35*0 ㎱
たんい #T35*0 ㎰
たんい #T35*0 ℃
たんい #T35*0 ℉
たんい #T35*0 ㏔
たんい #T35*0 ㏋
たんい #T35*0 ㎐
たんい #T35*0 ㎅
たんい #T35*0 ㎆
たんい #T35*0 ㎇
たんい #T35*0 TB
たんい #T35*0 ㍉
たんい #T35*0 ㌢
たんい #T35*0 ㍍
たんい #T35*0 ㌔
たんい #T35*0 ㌖
たんい #T35*0 ㌅
たんい #T35*0 ㌳
たんい #T35*0 ㍎
たんい #T35*0 ㌃
たんい #T35*0 ㌶
たんい #T35*0 ㌘
たんい #T35*0 ㌕
たんい #T35*0 ㌧
たんい #T35*0 ㍑
たんい #T35*0 ㍊
たんい #T35*0 ㌹
たんい #T35*0 ㍗
たんい #T35*0 ㌍
たんい #T35*0 ㍂
たんい #T35*0 ㌣
たんい #T35*0 ㌦
たんい #T35*0 ㌻
たんい #T35*0 ㌫
だくてん #T35*0 ゙
ちぇっくぼたん #T35*0 ☐
ちぇっくぼたん #T35*0 ☑
ちぇっくぼたん #T35*0 ☒
ちぇっくぼっくす #T35*0 ☐
ちぇっくぼっくす #T35*0 ☑
ちぇっくぼっくす #T35*0 ☒
てん #T35*0 、
てん #T35*0 。
てん #T35*0 ，
てん #T35*0 ．
てん #T35*0 ・
てん #T35*0 ：
てん #T35*0 ；
てん #T35*0 ？
てん #T35*0 ！
てん #T35*0 …
てん #T35*0 ‥
てん #T35*0 ゛
てん #T35*0 ゜
てん #T35*0 ´
てん #T35*0 ｀
てん #T35*0 ¨
てん #T35*0 ＾
てん #T35*0 °
てん #T35*0 ′
てん #T35*0 ″
てん #T35*0 ヽ
てん #T35*0 ヾ
てん #T35*0 ゝ
てん #T35*0 ゞ
てん #T35*0 〃
てん #T35*0 〟
てん #T35*0 〻
てん #T35*0 〳
てん #T35*0 〵
てん #T35*0 〴
てんすうじ #T35*0 ⒈
てんすうじ #T35*0 ⒉
てんすうじ #T35*0 ⒊
てんすうじ #T35*0 ⒋
てんすうじ #T35*0 ⒌
てんすうじ #T35*0 ⒍
てんすうじ #T35*0 ⒎
てんすうじ #T35*0 ⒏
てんすうじ #T35*0 ⒐
てんびんざ #T35*0 ♎
でんわ #T35*0 ℡
でんわ #T35*0 ☎
とらんぷ #T35*0 ♤
とらんぷ #T35*0 ♧
とらんぷ #T35*0 ♡
とらんぷ #T35*0 ♢
とらんぷ #T35*0 ♠
とらんぷ #T35*0 ♣
とらんぷ #T35*0 ♥
とらんぷ #T35*0 ♦
とれーどまーく #T35*0 ™
とん #T35*0 ㌧
ど #T35*0 ℃
ど #T35*0 °
ど #T35*0 ℉
どうのじ #T35*0 々
どる #T35*0 ＄
どる #T35*0 ㌦
なぜなら #T35*0 ∵
なんばー #T35*0 Ｎｏ
ねんごう #T35*0 ㍾
ねんごう #T35*0 ㍽
ねんごう #T35*0 ㍼
ねんごう #T35*0 ㍻
ののじ #T35*0 〃
はんだくてん #T35*0 ゚
ぱーせんと #T35*0 ㌫
ぱーせんと #T35*0 ％
ぱーみる #T35*0 ‰
びだくおん #T35*0 ゚
びだくおん #T35*0 ゜
びだくおん #T35*0 ŋ
びだくおん #T35*0 g̃
ふたござ #T35*0 ♊
ふらっと #T35*0 ♭
へいせい #T35*0 ㍻
へいほうきろ #T35*0 ㎢
へいほうせんち #T35*0 ㎠
へいほうみり #T35*0 ㎟
へいほうめーとる #T35*0 ㎡
ほし #T35*0 ★
ほし #T35*0 ☆
ほし #T35*0 ※
ぼたん #T35*0 ☐
ぼたん #T35*0 ☑
ぼたん #T35*0 ☒
ぽんど #T35*0 £
まる #T35*0 ◯
まる #T35*0 ●
まる #T35*0 ◎
まる #T35*0 〇
まる #T35*0 ○
まるもじ #T35*0 ①
まるもじ #T35*0 ②
まるもじ #T35*0 ③
まるもじ #T35*0 ④
まるもじ #T35*0 ⑤
まるもじ #T35*0 ⑥
まるもじ #T35*0 ⑦
まるもじ #T35*0 ⑧
まるもじ #T35*0 ⑨
まるもじ #T35*0 ⑩
まるもじ #T35*0 ⑪
まるもじ #T35*0 ⑫
まるもじ #T35*0 ⑬
まるもじ #T35*0 ⑭
まるもじ #T35*0 ⑮
まるもじ #T35*0 ⑯
まるもじ #T35*0 ⑰
まるもじ #T35*0 ⑱
まるもじ #T35*0 ⑲
まるもじ #T35*0 ⑳
まるもじ #T35*0 ❶
まるもじ #T35*0 ❷
まるもじ #T35*0 ❸
まるもじ #T35*0 ❹
まるもじ #T35*0 ❺
まるもじ #T35*0 ❻
まるもじ #T35*0 ❼
まるもじ #T35*0 ❽
まるもじ #T35*0 ❾
みぎさんかく #T35*0 ⊿
みずがめざ #T35*0 ♒
みり #T35*0 ㎜
みり #T35*0 ㍉
みりぐらむ #T35*0 ㎎
めいじ #T35*0 ㍾
めーとる #T35*0 m
めーとる #T35*0 ㍍
やぎざ #T35*0 ♑
やじるし #T35*0 →
やじるし #T35*0 ←
やじるし #T35*0 ↑
やじるし #T35*0 ↓
やじるし #T35*0 ⇒
やじるし #T35*0 ⇔
やじるし #T35*0 ☞
やじるし #T35*0 ☜
やじるし #T35*0 ☝
やじるし #T35*0 ☟
やじるし #T35*0 ⇆
やじるし #T35*0 ⇄
やじるし #T35*0 ⇅
やじるし #T35*0 ⇨
やじるし #T35*0 ⇦
やじるし #T35*0 ⇧
やじるし #T35*0 ⇩
ゆうびん #T35*0 〒
ゆうびん #T35*0 〠
ゆえに #T35*0 ∴
よこぼう #T35*0 ‐
よこぼう #T35*0 〓
よこぼう #T35*0 ￣
よこぼう #T35*0 ＿
よこぼう #T35*0 〜
よこぼう #T35*0 —
より #T35*0 ゟ
りっぽうせんち #T35*0 ㎤
りっぽうめーとる #T35*0 ㎥
ろーますうじ #T35*0 Ⅰ
ろーますうじ #T35*0 Ⅱ
ろーますうじ #T35*0 Ⅲ
ろーますうじ #T35*0 Ⅳ
ろーますうじ #T35*0 Ⅴ
ろーますうじ #T35*0 Ⅵ
ろーますうじ #T35*0 Ⅶ
ろーますうじ #T35*0 Ⅷ
ろーますうじ #T35*0 Ⅸ
ろーますうじ #T35*0 Ⅹ
ろーますうじ #T35*0 Ⅺ
ろーますうじ #T35*0 Ⅻ
ろーますうじ #T35*0 XIII
ろーますうじ #T35*0 XIV
ろーますうじ #T35*0 XV
ろーますうじ #T35*0 ⅰ
ろーますうじ #T35*0 ⅱ
ろーますうじ #T35*0 ⅲ
ろーますうじ #T35*0 ⅳ
ろーますうじ #T35*0 ⅴ
ろーますうじ #T35*0 ⅵ
ろーますうじ #T35*0 ⅶ
ろーますうじ #T35*0 ⅷ
ろーますうじ #T35*0 ⅸ
ろーますうじ #T35*0 ⅹ
ろーますうじ #T35*0 ⅺ
ろーますうじ #T35*0 ⅻ
ろーますうじ #T35*0 xiii
ろーますうじ #T35*0 xiv
ろーますうじ #T35*0 xv
