# vim:set noet ts=4:
#
# ibus-anthy - The Anthy engine for IBus
#
# Copyright (c) 2010-2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
# Copyright (c) 2010-2013 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# This file was imported from im-dict-ja
# im-dict-ja - Japanese word dictionary set for input method
# http://github.com/fujiwarat/im-dict-ja
#
# Typing 'va' can convert 'ヷ' with this dictionary.
う゛ぁ #T35*500 ヷ
う゛ぃ #T35*500 ヸ
う゛ぇ #T35*500 ヹ
う゛ぉ #T35*500 ヺ
# 〳, 〵and 〴 can be used instead.
おなじ #T35*500 〱
おなじ #T35*500 〲
くのじ #T35*500 〱
くのじ #T35*500 〲
くりかえし #T35*500 〱
くりかえし #T35*500 〲
てん #T35*500 〱
てん #T35*500 〲
